from typing import Optional


def validate_api_key(api_key: Optional[str] = None) -> None:
    """
    License validation is no longer required.
    This function is kept for backward compatibility but does nothing.
    """
    pass
