from django.apps import AppConfig


class PaymeConfig(AppConfig):
    """Payme app config."""
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'payme'

    def ready(self):
        pass
